import { Observable } from '@nativescript/core';
import { BehaviorSubject } from "rxjs";
export class Common extends Observable {
    constructor() {
        super(...arguments);
        this.isConnected = new BehaviorSubject(null);
        this._status = new BehaviorSubject(null);
        this._getStatus = { name: 'No Start', id: -1 };
        this.methods = {};
        this.callbacks = {};
        this.recordSeparator = String.fromCharCode(0x1e);
        this.roomId = 0;
        this.socketUrl = '';
        this.protocol = {
            name: 'json',
            transferFormat: 1,
            version: 1,
            writeMessage: (message) => {
                return TextMessageFormat.write(JSON.stringify(message));
            }
        };
    }
    setStatus(status) {
        this._status.next(status);
        this._getStatus = status;
    }
    getStatus$() {
        return this._status.asObservable();
    }
    getStatus() {
        return this._getStatus;
    }
    start(httpURL, header) {
        return new Promise((resolve, reject) => {
            const run = () => {
                let queryString = "?";
                let idParam = "id=";
                let urlPieces = httpURL.split("?");
                if (urlPieces.length > 1) {
                    queryString += urlPieces[1];
                    httpURL = urlPieces[0];
                    idParam = "&" + idParam;
                }
                this.socketUrl = httpURL.replace(/(http)(s)?\:\/\//, 'ws$2://');
                this.socketUrl += queryString + idParam;
                const self = this;
                this.makeRequest(header, 'POST', `${httpURL}/negotiate` + queryString, (err, data) => {
                    this.setStatus({ id: 0, name: 'Start negotiate' });
                    if (err) {
                        reject(err);
                        this.setStatus({ id: -1, name: 'Error negotiate' });
                        return false;
                    }
                    else {
                        let connId = this.socketUrl;
                        this.setStatus({ id: 1, name: 'Ok, negotiate' });
                        if (typeof data === 'object') {
                            connId = data.connectionId;
                        }
                        else {
                            const _data = JSON.parse(data);
                            connId = _data.connectionId;
                        }
                        this.socketUrl += connId;
                        return self.openSocketConnection(this.socketUrl)
                            .then((res) => {
                            this.setStatus({ id: 2, name: 'Socket Opened' });
                            if (res) {
                                this.isConnected.next(true);
                                resolve(true);
                                return true;
                            }
                        });
                    }
                });
            };
            if (this.websocket && this.websocket.readyState && this.websocket.readyState !== WebSocketStatus.Closed) {
                this.close();
                reject('Error');
                return run();
            }
            else {
                return run();
            }
        });
    }
    openSocketConnection(socketUrl) {
        return new Promise((resolve, reject) => {
            this.websocket = new WebSocket(socketUrl, this.recordSeparator[JSON.stringify(this.protocol)]);
            this.websocket.onopen = (event) => {
                this.websocket.send(JSON.stringify({ 'protocol': 'json', 'version': 1 }));
                this.websocket.send(this.recordSeparator);
                if (this.methods["connected"]) {
                    this.methods["connected"][0]();
                }
            };
            this.websocket.onmessage = (data) => this._onMessage(data);
            this.websocket.onclose = () => {
                if (this.methods["disconnected"]) {
                    this.methods["disconnected"][0]();
                }
                this.close();
            };
            this.websocket.onerror = (err) => reject(err);
            return resolve(this.websocket);
        });
    }
    close() {
        return new Promise((resolve, reject) => {
            if (this.websocket && this.websocket.readyState !== WebSocketStatus.Closed) {
                this.websocket.close();
                resolve('SignalR - Connection closed!');
                this.setStatus({ id: -1, name: 'Socket closed!' });
            }
            reject('Error - Connection already close!');
        });
    }
    on(methodName, newMethod) {
        if (!methodName || !newMethod) {
            return;
        }
        methodName = methodName.toLowerCase();
        if (!this.methods[methodName]) {
            this.methods[methodName] = [];
        }
        if (this.methods[methodName].indexOf(newMethod) !== -1) {
            return;
        }
        this.methods[methodName].push(newMethod);
    }
    invoke(methodName, ...args) {
        for (let _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        const invocationDescriptor = this.createInvocation(methodName, args, false);
        let _invocationEvent = null;
        return new Promise((resolve, reject) => {
            if (!this.websocket) {
                return reject('Error - did you call start(hubUrl) ?');
            }
            if (this.websocket && this.websocket.readyState === 3) {
                return reject('Error - socket is not connected');
            }
            this.callbacks[invocationDescriptor.invocationId] = (invocationEvent, error) => {
                if (error) {
                    reject(error);
                    return;
                }
                else if (invocationEvent) {
                    if (invocationEvent.type === MessageType.Completion) {
                        if (invocationEvent.error) {
                            reject(new Error(invocationEvent.error));
                        }
                        else {
                            resolve(invocationEvent.result);
                        }
                    }
                    else {
                        reject(new Error('Unexpected message type: ' + invocationEvent.type));
                    }
                }
            };
            const message = this.protocol.writeMessage(invocationDescriptor);
            this.websocket.send(message);
            resolve({ data: TextMessageFormat.parse(message), date: new Date() });
        });
    }
    createInvocation(methodName, args, nonblocking) {
        if (nonblocking) {
            return {
                arguments: args,
                target: methodName,
                type: MessageType.Invocation,
            };
        }
        else {
            const id = this.roomId;
            this.roomId++;
            return {
                arguments: args,
                invocationId: id.toString(),
                target: methodName,
                type: MessageType.Invocation,
            };
        }
    }
    _onMessage(event) {
        const { data, target } = event;
        const parseData = TextMessageFormat.parse(data);
        const parseDataList = TextMessageFormat.parse(data);
        for (var i in parseDataList) {
            const parseTarget = JSON.parse(parseDataList[i]);
            if (parseTarget && parseTarget['type']) {
                if (parseTarget['type'] === MessageType.Completion && parseTarget.invocationId && this.callbacks[parseTarget.invocationId]) {
                    this.callbacks[parseTarget.invocationId](parseTarget, parseTarget.error);
                }
                else if (parseTarget['type'] === MessageType.Invocation && parseTarget.target && this.methods[parseTarget.target.toLowerCase()]) {
                    this.methods[parseTarget.target.toLowerCase()][0](parseTarget);
                }
            }
        }
    }
    makeRequest(header, method, url, done) {
        const xhr = new XMLHttpRequest();
        xhr.open(method, url);
        if (header) {
            xhr.setRequestHeader(header.key, header.value);
        }
        xhr.onload = function () {
            done(null, xhr.response);
        };
        xhr.onerror = function (err) {
            done(xhr.response || err);
        };
        xhr.send();
    }
}
export var WebSocketStatus;
(function (WebSocketStatus) {
    WebSocketStatus[WebSocketStatus["Connecting"] = 0] = "Connecting";
    WebSocketStatus[WebSocketStatus["Open"] = 1] = "Open";
    WebSocketStatus[WebSocketStatus["Closing"] = 2] = "Closing";
    WebSocketStatus[WebSocketStatus["Closed"] = 3] = "Closed";
})(WebSocketStatus || (WebSocketStatus = {}));
export var MessageType;
(function (MessageType) {
    MessageType[MessageType["Invocation"] = 1] = "Invocation";
    MessageType[MessageType["StreamItem"] = 2] = "StreamItem";
    MessageType[MessageType["Completion"] = 3] = "Completion";
    MessageType[MessageType["StreamInvocation"] = 4] = "StreamInvocation";
    MessageType[MessageType["CancelInvocation"] = 5] = "CancelInvocation";
    MessageType[MessageType["Ping"] = 6] = "Ping";
    MessageType[MessageType["Close"] = 7] = "Close";
})(MessageType || (MessageType = {}));
export class TextMessageFormat {
    static write(output) {
        return `${output}${TextMessageFormat.RecordSeparator}`;
    }
    static parse(input) {
        if (input[input.length - 1] !== TextMessageFormat.RecordSeparator) {
            throw new Error('incomplete.');
        }
        const messages = input.split(TextMessageFormat.RecordSeparator);
        messages.pop();
        return messages;
    }
}
TextMessageFormat.RecordSeparatorCode = 0x1e;
TextMessageFormat.RecordSeparator = String.fromCharCode(TextMessageFormat.RecordSeparatorCode);
export class Status {
}
export class SignalRCoreRHeaders {
    constructor(key, value) {
        this.key = key;
        this.value = value;
    }
}
//# sourceMappingURL=signalr-core.common.js.map